//
//  YKLookSwingBase
//  YKLookComponent
//  YKLookLabel
//  YKLookButton
//  YKLookToggleButton
//  YKLookCheckBox
//  YKLookRadioButton
//  YKLookButtonGroup
//  YKLookList
//  YKLookPopupList
//  YKLookComboBox
//  YKLookPasswordField
//  YKLookTextField
//  YKLookTextArea
//  YKLookCursor
//  YKLookImage
//  YKLookColor
//  YKLookFont
//  YKLookPanelFlow
//  YKLookPanelBox
//  YKLookPanelGrid
//  YKLookPanelGridBag
//  YKLookPanelCard
//  YKLookPanelQuint
//  YKLookBoxFiller
//
//  Copyright (c) 2009-2010 YorozuKotowariYa Ltd.
//
//
package jp.co.yorozukotowariya.yklook;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

class YKLookSwingBase
{
	static final String[] ykvCursorString = { // "CUSTOM",
								"DEFAULT"  , "CROSSHAIR",
								"TEXT"     , "WAIT"     ,
								"SW_RESIZE", "SE_RESIZE",
								"NW_RESIZE", "NE_RESIZE",
								"N_RESIZE" , "S_RESIZE" ,
								"W_RESIZE" , "E_RESIZE" ,
								"HAND"     , "MOVE"     };
	static final int[] ykvCursorInt = { // Cursor.CUSTOM_CURSOR,
							Cursor.DEFAULT_CURSOR  , Cursor.CROSSHAIR_CURSOR,
							Cursor.TEXT_CURSOR     , Cursor.WAIT_CURSOR     ,
							Cursor.SW_RESIZE_CURSOR, Cursor.SE_RESIZE_CURSOR,
							Cursor.NW_RESIZE_CURSOR, Cursor.NE_RESIZE_CURSOR,
							Cursor.N_RESIZE_CURSOR , Cursor.S_RESIZE_CURSOR ,
							Cursor.W_RESIZE_CURSOR , Cursor.E_RESIZE_CURSOR ,
							Cursor.HAND_CURSOR     , Cursor.MOVE_CURSOR     };

	int          ykvFrame;
	Integer      ykvId;
	YKLookCommon ykvCommon;

	Cursor ykmGetCursor( String prmString )
	{
		String       str = prmString.substring(2);
		YKLookCursor cur;
		int          idx;

		if( prmString.charAt(0) == 'o' )
		{
			cur = (YKLookCursor) ykvCommon.ykmFindObjectFromId( str );
			return cur.ykvSelf;
		}
		str = str.toUpperCase();

		for( idx = 0; idx < ykvCursorString.length; idx++ )
		{
			if( str.equalsIgnoreCase(ykvCursorString[idx]) )
			{
				return Cursor.getPredefinedCursor( ykvCursorInt[idx] );
			}
		}
		return null;
	}
	Dimension ykmDimension( StringTokenizer prmStrTok )
	{
		int width  = Integer.parseInt( prmStrTok.nextToken() );
		int height = Integer.parseInt( prmStrTok.nextToken() );
		return new Dimension( width, height );
	}
	Dimension ykmDimension( StringTokenizer prmStrTok, Dimension prmDefault )
	{
		String strWidth  = prmStrTok.nextToken();
		String strHeight = prmStrTok.nextToken();
		Dimension    dim = ykmDimension( prmStrTok );
		int    w,h;

		switch( strWidth.charAt(0) )
		{
			case 'e':
			case 'E':
			case 'n':
			case 'N':
					if( prmDefault == null ) { w = dim       .width; }
					else                     { w = prmDefault.width; }
					break;
			default:
					w = Integer.parseInt( strWidth );
					break;
		}
		switch( strHeight.charAt(0) )
		{
			case 'e':
			case 'E':
			case 'n':
			case 'N':
					if( prmDefault == null ) { h = dim       .height; }
					else                     { h = prmDefault.height; }
					break;
			default:
					h = Integer.parseInt( strHeight );
					break;
		}
		return new Dimension( w, h );
	}
	Object ykmEncodeProperty( String prmStr )
	{
		String dat = prmStr.substring(2);

		switch( prmStr.charAt(0) )
		{
			case 'i': return new Integer( dat );
			case 'S': return ykvCommon.ykmEncodingFrom(
										YKLookCommon.ykmDecodeFromHex( dat ) );
		}
		if( prmStr.equalsIgnoreCase( "s/true" ) )
			return Boolean.TRUE;
		if( prmStr.equalsIgnoreCase( "s/false" ) )
			return Boolean.FALSE;

		return null;
	}
}
class YKLookReturnable extends YKLookSwingBase implements YKLookDumpIF
{
	String ykvCallback = null;

	public void ykmDumpGuiInfo( StringBuffer prmBuf )
			throws YKLookExceptionEncoding
	{
		// nothing
	}

	void ykmSetCallback( StringTokenizer prmStrTok )
	{
		ykvCallback = ykvCommon.ykmNextString( prmStrTok );
		if( ykvCallback != null )
			if( ykvCallback.length() == 0 )
				ykvCallback = null;
	}

	String ykmMakeEventHeader( String prmHeader )
	{
		String cb;
		int pos;

		if( ykvCallback == null )
			return null;

		if( ykvCallback.length() <= 0 )
			return null;

		try
		{
			cb = ykvCommon.ykmDumpString( ykvCallback );
			return prmHeader + ":" + ykvId.toString() +
								":" + ykvFrame + ":" + cb;
		}
		catch( YKLookExceptionEncoding ex )
		{
			cb  = getClass().getName();
			pos = cb.lastIndexOf( '.' );
			cb  = "illegal callback method name (" +
					cb.substring( pos + 1 ) + ")";

			JOptionPane.showMessageDialog( null, cb, "alert",
										JOptionPane.ERROR_MESSAGE );
		}
		return null;
	}
}
abstract class YKLookElement extends YKLookReturnable
{
	YKLookElement ykmCheckClone()
	{
		Component co = ykmComponent();

		if( co != null )
		{
			if( co.getParent() == null )
				return this;
		}
		return null;
	}
	abstract Component ykmComponent();  // need override
}
class YKLookComponent extends YKLookElement
{
	JComponent ykvComponent;

	Component ykmComponent()  // override
	{
		return ykvComponent;
	}
	void ykmSetMinimumSizeIfNeed( int prmWidth, int prmHeight )
	{
		if( ! ykvComponent.isMinimumSizeSet() )
			ykvComponent.setMinimumSize( new Dimension( prmWidth, prmHeight ) );
	}

	float ykmAlign( StringTokenizer prmStrTok )
	{
		String tok = prmStrTok.nextToken();
		float  align;

		switch( tok.charAt(0) )
		{
			case 'l': case 'L': return Component.LEFT_ALIGNMENT;
			case 'r': case 'R': return Component.RIGHT_ALIGNMENT;
			case 't': case 'T': return Component.TOP_ALIGNMENT;
			case 'b': case 'B': return Component.BOTTOM_ALIGNMENT;
			case 'c': case 'C': return Component.CENTER_ALIGNMENT;
		}
		try
		{
			return Float.parseFloat( tok ) / 100.0f;
		}
		catch( Exception ex )
		{
		}
		return Component.CENTER_ALIGNMENT;
	}
	static int ykmHorizontal( StringTokenizer prmStrTok ) throws YKLookException
	{
		String tok = prmStrTok.nextToken().substring(0,3).toLowerCase();

		if( tok.equals("lef") ) return SwingConstants.LEFT;
		if( tok.equals("cen") ) return SwingConstants.CENTER;
		if( tok.equals("rig") ) return SwingConstants.RIGHT;
		if( tok.equals("lea") ) return SwingConstants.LEADING;
		if( tok.equals("tra") ) return SwingConstants.TRAILING;

		throw new YKLookException("horizontal");
	}
	static int ykmVertical( StringTokenizer prmStrTok )
	{
		String tok = prmStrTok.nextToken();

		switch( tok.charAt(0) )
		{
			case 't': case 'T': return SwingConstants.TOP;
			case 'b': case 'B': return SwingConstants.BOTTOM;
			//case 'c': case 'C': return SwingConstants.CENTER;
		}
		return SwingConstants.CENTER;
	}
	Color ykmDecodeColorOrNull( StringTokenizer prmStrTok )
	{
		String tok = prmStrTok.nextToken();

		if( tok.equalsIgnoreCase( "n/null" ) || tok.equalsIgnoreCase( "null" ) )
			return null;

		return ykvCommon.ykmDecodeColor( tok );
	}

	public boolean yk_requestFocus( StringTokenizer prmStrTok )
	{
		ykvComponent.requestFocus();
		return true;  // need scan for 'end'
	}
	public boolean yk_putClientProperty( StringTokenizer prmStrTok )
	{
		Object key = ykmEncodeProperty( prmStrTok.nextToken() );
		Object val = ykmEncodeProperty( prmStrTok.nextToken() );

		ykvComponent.putClientProperty( key, val );
		return true;  // need scan for 'end'
	}
	public boolean yk_setMinimumSize( StringTokenizer prmStrTok )
	{
		Dimension dim = ykvComponent.getMinimumSize();
		dim = ykmDimension( prmStrTok, dim );
		ykvComponent.setMinimumSize( dim );
		return true;  // need scan for 'end'
	}
	public boolean yk_setMaximumSize( StringTokenizer prmStrTok )
	{
		Dimension dim = ykvComponent.getMaximumSize();
		dim = ykmDimension( prmStrTok, dim );
		ykvComponent.setMaximumSize( dim );
		return true;  // need scan for 'end'
	}
	public boolean yk_setPreferredSize( StringTokenizer prmStrTok )
	{
		Dimension dim = ykvComponent.getPreferredSize();
		dim = ykmDimension( prmStrTok, dim );
		ykvComponent.setPreferredSize( dim );
		return true;  // need scan for 'end'
	}
	public boolean yk_fixSize( StringTokenizer prmStrTok )
	{
		Dimension dim = ykvComponent.getPreferredSize();
		dim = ykmDimension( prmStrTok, dim );

		ykvComponent.setMinimumSize( dim );
		ykvComponent.setPreferredSize( dim );
		ykvComponent.setMaximumSize( dim );
		return true;  // need scan for 'end'
	}
	public boolean yk_setAlignmentX( StringTokenizer prmStrTok )
	{
		ykvComponent.setAlignmentX( ykmAlign( prmStrTok ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setAlignmentY( StringTokenizer prmStrTok )
	{
		ykvComponent.setAlignmentY( ykmAlign( prmStrTok ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setEnabled( StringTokenizer prmStrTok )
	{
		char chr = prmStrTok.nextToken().charAt(0);

		ykvComponent.setEnabled( chr == 'T' || chr == 't' );

		return true;  // need scan for 'end'
	}
	public boolean yk_setVisible( StringTokenizer prmStrTok )
	{
		char chr = prmStrTok.nextToken().charAt(0);

		ykvComponent.setVisible( chr == 'T' || chr == 't' );

		return true;  // need scan for 'end'
	}
	public boolean yk_setFont( StringTokenizer prmStrTok )
	{
		String     tok = prmStrTok.nextToken();
		YKLookFont fnt;

		fnt = (YKLookFont) ykvCommon.ykmFindObjectFromId( tok );
		if( fnt == null )
			ykvComponent.setFont( null );
		else
			ykvComponent.setFont( fnt.ykvSelf );

		return true;  // need scan for 'end'
	}
	public boolean yk_setForeground( StringTokenizer prmStrTok )
	{
		ykvComponent.setForeground( ykmDecodeColorOrNull( prmStrTok ) );

		return true;  // need scan for 'end'
	}
	public boolean yk_setBackground( StringTokenizer prmStrTok )
	{
		ykvComponent.setBackground( ykmDecodeColorOrNull( prmStrTok ) );

		return true;  // need scan for 'end'
	}
	public boolean yk_setToolTipText( StringTokenizer prmStrTok )
	{
		String txt = ykvCommon.ykmNextString( prmStrTok );

		ykvComponent.setToolTipText( txt );

		return true;  // need scan for 'end'
	}
	public boolean yk_setBorder( StringTokenizer prmStrTok )
	{
		String       tok = prmStrTok.nextToken();
		YKLookBorder bdr;

		bdr = (YKLookBorder) ykvCommon.ykmFindObjectFromId( tok );
		ykvComponent.setBorder( bdr.ykvSelf );
		return true;  // need scan for 'end'
	}
	public boolean yk_setCursor( StringTokenizer prmStrTok )
	{
		ykvComponent.setCursor( ykmGetCursor( prmStrTok.nextToken() ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setOpaque( StringTokenizer prmStrTok )
	{
		char chr = prmStrTok.nextToken().charAt(0);

		ykvComponent.setOpaque( chr == 'T' || chr == 't' );
		return true;  // need scan for 'end'
	}
	void ykmSendEventAction( ActionEvent prmEvent )
	{
		int          mod  = prmEvent.getModifiers();
		StringBuffer keys = new StringBuffer();
		String       head = ykmMakeEventHeader( "YKLookEventAction" );

		if( head == null )
			return;

		if( (mod & ActionEvent.SHIFT_MASK) != 0 ) { keys.append(":SHIFT"); }
		if( (mod & ActionEvent.CTRL_MASK ) != 0 ) { keys.append(":CTRL" ); }
		if( (mod & ActionEvent.META_MASK ) != 0 ) { keys.append(":META" ); }
		if( (mod & ActionEvent.ALT_MASK  ) != 0 ) { keys.append(":ALT"  ); }

		ykvCommon.ykmSendEventAny( head + keys.toString() + ":end:" );
	}
	static String ykmModifierKeys( MouseEvent prmEvent )
	{
		int          mod  = prmEvent.getModifiers();
		StringBuffer keys = new StringBuffer();

		if((mod & InputEvent.SHIFT_MASK    ) !=0) { keys.append(":SHIFT"    ); }
		if((mod & InputEvent.CTRL_MASK     ) !=0) { keys.append(":CTRL"     ); }
		if((mod & InputEvent.META_MASK     ) !=0) { keys.append(":META"     ); }
		if((mod & InputEvent.ALT_MASK      ) !=0) { keys.append(":ALT"      ); }
		if((mod & InputEvent.ALT_GRAPH_MASK) !=0) { keys.append(":ALT_GRAPH"); }
		if((mod & InputEvent.BUTTON1_MASK  ) !=0) { keys.append(":BUTTON1"  ); }
		if((mod & InputEvent.BUTTON2_MASK  ) !=0) { keys.append(":BUTTON2"  ); }
		if((mod & InputEvent.BUTTON3_MASK  ) !=0) { keys.append(":BUTTON3"  ); }

		return keys.toString();
	}
}
class YKLookScrollable extends YKLookComponent
{
	YKLookComponent ykvScrollPane = null;

	YKLookElement ykmCheckClone()
	{
		if( ykvScrollPane == null )
			return super.ykmCheckClone();

		return ( ykvScrollPane.ykmCheckClone() == ykvScrollPane ) ?
													ykvScrollPane : null;
	}
	void ykmSetMinimumSizeIfNeed( int prmWidth, int prmHeight )
	{
		if( ykvScrollPane == null )
			super.ykmSetMinimumSizeIfNeed( prmWidth, prmHeight );
		else
			ykvScrollPane.ykmSetMinimumSizeIfNeed( prmWidth, prmHeight );
	}
	public boolean yk_setScrollPane( StringTokenizer prmStrTok )
	{
		ykvScrollPane = new YKLookComponent();
		ykvScrollPane.ykvComponent = new JScrollPane( ykvComponent );
		return true;  // need scan for 'end'
	}
	public boolean yk_setMinimumSize( StringTokenizer prmStrTok )
	{
		if( ykvScrollPane == null )
			return super.yk_setMinimumSize( prmStrTok );

		return ykvScrollPane.yk_setMinimumSize( prmStrTok );
	}
	public boolean yk_setMaximumSize( StringTokenizer prmStrTok )
	{
		if( ykvScrollPane == null )
			return super.yk_setMaximumSize( prmStrTok );

		return ykvScrollPane.yk_setMaximumSize( prmStrTok );
	}
	public boolean yk_setPreferredSize( StringTokenizer prmStrTok )
	{
		if( ykvScrollPane == null )
			return super.yk_setPreferredSize( prmStrTok );

		return ykvScrollPane.yk_setPreferredSize( prmStrTok );
	}
	public boolean yk_fixSize( StringTokenizer prmStrTok )
	{
		if( ykvScrollPane == null )
			return super.yk_fixSize( prmStrTok );

		return ykvScrollPane.yk_fixSize( prmStrTok );
	}
	public boolean yk_setAlignmentX( StringTokenizer prmStrTok )
	{
		if( ykvScrollPane == null )
			return super.yk_setAlignmentX( prmStrTok );

		return ykvScrollPane.yk_setAlignmentX( prmStrTok );
	}
	public boolean yk_setAlignmentY( StringTokenizer prmStrTok )
	{
		if( ykvScrollPane == null )
			return super.yk_setAlignmentY( prmStrTok );

		return ykvScrollPane.yk_setAlignmentY( prmStrTok );
	}
	public boolean yk_setEnabled( StringTokenizer prmStrTok )
	{
		if( ykvScrollPane == null )
			return super.yk_setEnabled( prmStrTok );

		return ykvScrollPane.yk_setEnabled( prmStrTok );
	}
	public boolean yk_setVisible( StringTokenizer prmStrTok )
	{
		if( ykvScrollPane == null )
			return super.yk_setVisible( prmStrTok );

		return ykvScrollPane.yk_setVisible( prmStrTok );
	}
}
class YKLookLabel extends YKLookComponent
{
	JLabel ykvSelf;

	YKLookLabel()
	{
		ykvComponent = ykvSelf = new JLabel();
	}

	YKLookElement ykmCheckClone()
	{
		YKLookLabel co;
		JLabel      lab;

		if( super.ykmCheckClone() == this )
			return this;

		co  = new YKLookLabel();
		lab = co.ykvSelf;

		lab.setText( ykvSelf.getText() );
		lab.setIcon( ykvSelf.getIcon() );
		lab.setToolTipText( ykvSelf.getToolTipText() );

		if( ykvSelf.isMinimumSizeSet() )
			lab.setMinimumSize( ykvSelf.getMinimumSize() );
		if( ykvSelf.isMaximumSizeSet() )
			lab.setMaximumSize( ykvSelf.getMaximumSize() );
		if( ykvSelf.isPreferredSizeSet() )
			lab.setPreferredSize( ykvSelf.getPreferredSize() );

		lab.setAlignmentX( ykvSelf.getAlignmentX() );
		lab.setAlignmentY( ykvSelf.getAlignmentY() );
		lab.setVerticalAlignment( ykvSelf.getVerticalAlignment() );
		lab.setHorizontalAlignment( ykvSelf.getHorizontalAlignment() );
		lab.setVerticalTextPosition( ykvSelf.getVerticalTextPosition() );
		lab.setHorizontalTextPosition( ykvSelf.getHorizontalTextPosition() );
		return co;
	}

	public boolean yk_setText( StringTokenizer prmStrTok )
	{
		String txt = ykvCommon.ykmNextString( prmStrTok );

		ykvSelf.setText( txt );

		return true;  // need scan for 'end'
	}
	public boolean yk_setIcon( StringTokenizer prmStrTok )
	{
		String      tok = prmStrTok.nextToken();
		YKLookImage img;

		img = (YKLookImage) ykvCommon.ykmFindObjectFromId( tok );
		if( img == null )
			ykvSelf.setIcon( null );
		else
			ykvSelf.setIcon( img.ykvSelf );

		return true;  // need scan for 'end'
	}
	public boolean yk_setVerticalAlignment( StringTokenizer prmStrTok )
	{
		ykvSelf.setVerticalAlignment( ykmVertical( prmStrTok ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setHorizontalAlignment( StringTokenizer prmStrTok )
			throws YKLookException
	{
		ykvSelf.setHorizontalAlignment( ykmHorizontal( prmStrTok ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setVerticalTextPosition( StringTokenizer prmStrTok )
	{
		ykvSelf.setVerticalTextPosition( ykmVertical( prmStrTok ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setHorizontalTextPosition( StringTokenizer prmStrTok )
			throws YKLookException
	{
		ykvSelf.setHorizontalTextPosition( ykmHorizontal( prmStrTok ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setTextPosition( StringTokenizer prmStrTok )
	{
		switch( prmStrTok.nextToken().charAt(0) )
		{
			case '0':
			case 'R': // right
			case 'r':
			default:
					ykvSelf.setVerticalTextPosition  ( SwingConstants.CENTER );
					ykvSelf.setHorizontalTextPosition( SwingConstants.RIGHT  );
					break;
			case '1':
			case 'L': // left
			case 'l':
					ykvSelf.setVerticalTextPosition  ( SwingConstants.CENTER );
					ykvSelf.setHorizontalTextPosition( SwingConstants.LEFT   );
					break;
			case '2':
			case 'B': // bottom
			case 'b':
					ykvSelf.setVerticalTextPosition  ( SwingConstants.BOTTOM );
					ykvSelf.setHorizontalTextPosition( SwingConstants.CENTER );
					break;
			case '3':
			case 'T': // top
			case 't':
					ykvSelf.setVerticalTextPosition  ( SwingConstants.TOP    );
					ykvSelf.setHorizontalTextPosition( SwingConstants.CENTER );
					break;
		}
		return true;  // need scan for 'end'
	}
}
class YKLookAbstractButton extends YKLookComponent implements ActionListener
{
	AbstractButton ykvSelf;

	public boolean yk_setText( StringTokenizer prmStrTok )
	{
		String txt = ykvCommon.ykmNextString( prmStrTok );

		ykvSelf.setText( txt );

		return true;  // need scan for 'end'
	}
	public boolean yk_setIcon( StringTokenizer prmStrTok )
	{
		String      tok = prmStrTok.nextToken();
		YKLookImage img;

		img = (YKLookImage) ykvCommon.ykmFindObjectFromId( tok );
		if( img == null )
			ykvSelf.setIcon( null );
		else
			ykvSelf.setIcon( img.ykvSelf );

		return true;  // need scan for 'end'
	}
	public boolean yk_setVerticalAlignment( StringTokenizer prmStrTok )
	{
		ykvSelf.setVerticalAlignment( ykmVertical( prmStrTok ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setHorizontalAlignment( StringTokenizer prmStrTok )
			throws YKLookException
	{
		ykvSelf.setHorizontalAlignment( ykmHorizontal( prmStrTok ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setVerticalTextPosition( StringTokenizer prmStrTok )
	{
		ykvSelf.setVerticalTextPosition( ykmVertical( prmStrTok ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setHorizontalTextPosition( StringTokenizer prmStrTok )
			throws YKLookException
	{
		ykvSelf.setHorizontalTextPosition( ykmHorizontal( prmStrTok ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setTextPosition( StringTokenizer prmStrTok )
	{
		switch( prmStrTok.nextToken().charAt(0) )
		{
			case '0':
			case 'R': // right
			case 'r':
			default:
					ykvSelf.setVerticalTextPosition  ( SwingConstants.CENTER );
					ykvSelf.setHorizontalTextPosition( SwingConstants.RIGHT  );
					break;
			case '1':
			case 'L': // left
			case 'l':
					ykvSelf.setVerticalTextPosition  ( SwingConstants.CENTER );
					ykvSelf.setHorizontalTextPosition( SwingConstants.LEFT   );
					break;
			case '2':
			case 'B': // bottom
			case 'b':
					ykvSelf.setVerticalTextPosition  ( SwingConstants.BOTTOM );
					ykvSelf.setHorizontalTextPosition( SwingConstants.CENTER );
					break;
			case '3':
			case 'T': // top
			case 't':
					ykvSelf.setVerticalTextPosition  ( SwingConstants.TOP    );
					ykvSelf.setHorizontalTextPosition( SwingConstants.CENTER );
					break;
		}
		return true;  // need scan for 'end'
	}
	public boolean yk_setSelected( StringTokenizer prmStrTok )
	{
		char ch = prmStrTok.nextToken().charAt(0);

		ykvSelf.setSelected( ( ch == 't' || ch == 'T' ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setListener( StringTokenizer prmStrTok )
	{
		ykmSetCallback( prmStrTok );

		ykvSelf.removeActionListener( this );

		if( ykvCallback != null || ykvCommon.ykmHasNotification( this ) )
			ykvSelf.addActionListener( this );

		return true;  // need scan for 'end'
	}

	public void actionPerformed( ActionEvent prmEvent )
	{
		ykvCommon.ykmSendNotification( this, prmEvent );

		ykmSendEventAction( prmEvent );
	}
}
class YKLookButton extends YKLookAbstractButton
{
	YKLookButton()
	{
		ykvComponent = ykvSelf = new JButton();
		ykvSelf.addActionListener( this );
	}
}
class YKLookToggleButton extends YKLookAbstractButton
{
	YKLookToggleButton()
	{
		this( new JToggleButton() );
	}
	YKLookToggleButton( JToggleButton prmButton )
	{
		ykvComponent = ykvSelf = prmButton;
	}

	public void ykmDumpGuiInfo( StringBuffer prmBuf )
	{
		String tf = ykvSelf.isSelected() ? "true" : "false";

		prmBuf.append( ykvId.toString() + ":" + tf + ":end:" );
	}
}
class YKLookCheckBox extends YKLookToggleButton
{
	YKLookCheckBox()
	{
		super( new JCheckBox() );
	}
}
class YKLookRadioButton extends YKLookToggleButton
{
	YKLookRadioButton()
	{
		super( new JRadioButton() );
	}
}
class YKLookButtonGroup extends YKLookSwingBase
{
	ButtonGroup ykvSelf;

	YKLookButtonGroup()
	{
		ykvSelf = new ButtonGroup();
	}
	public boolean yk_add( StringTokenizer prmStrTok )
	{
		int                  len = Integer.parseInt( prmStrTok.nextToken() );
		int                  idx;
		String               tok;
		YKLookAbstractButton btn;

		for( idx = 0; idx < len; idx++ )
		{
			tok = prmStrTok.nextToken();
			btn = (YKLookAbstractButton) ykvCommon.ykmFindObjectFromId(tok);
			if( btn != null )
			{
				ykvSelf.add( btn.ykvSelf );
			}
		}
		return true;  // need scan for 'end'
	}
}
class YKLookListCellRendererJ extends JLabel implements ListCellRenderer
{
	Dimension ykvSize = new Dimension();

	YKLookListCellRendererJ()
	{
		setOpaque( true );
	}
	public Component getListCellRendererComponent(  JList   prmList,
													Object  prmItem,
													int     prmIndex,
													boolean prmIsSelected,
													boolean prmCellHasFocus )
	{
		Component co = (Component) prmItem;

		if( prmItem instanceof String )
		{
			setText( (String) prmItem );
			setIcon(          null    );
			co = this;
		}
		if( prmItem instanceof Icon )
		{
			setIcon( (Icon) prmItem );
			setText(        null    );
			co = this;
		}
		if( prmIsSelected )
		{
			co.setForeground( prmList.getSelectionForeground() );
			co.setBackground( prmList.getSelectionBackground() );
		}
		else
		{
			co.setForeground( prmList.getForeground() );
			co.setBackground( prmList.getBackground() );
		}
		return co;
	}

	public void setPreferredSize( Dimension prmSize )  // override JLabel
	{
		if( ykvSize.width < prmSize.width || ykvSize.height < prmSize.height )
		{
			ykvSize.width  = prmSize.width;
			ykvSize.height = prmSize.height;
			super.setPreferredSize( ykvSize );
		}
	}
}
class YKLookList extends YKLookScrollable implements MouseListener
{
	JList                   ykvSelf;
	DefaultListModel        ykvModel      = new DefaultListModel();
	YKLookListCellRendererJ ykvRender     = null;
	int                     ykvClickCount = 1;
	long                    ykvMouseWhen  = 0;
	ArrayList               ykvItems      = new ArrayList();

	YKLookList()
	{
		ykvComponent = ykvSelf = new JList( ykvModel );
	}

	Object ykmDecodeItem( String prmToken )
	{
		char   typ = prmToken.charAt(0);
		String dat = prmToken.substring(2);
		Object obj;
		JLabel lab;

		if( typ == 'S' )
		{
			return ykvCommon.ykmEncodingFrom(
						YKLookCommon.ykmDecodeFromHex( dat ) );
		}
		if( typ == 'i' || typ == 'd' )
		{
			return dat;
		}

		obj = ykvCommon.ykmFindObjectFromId( dat );
		if( obj instanceof YKLookImage )
		{
			return ((YKLookImage) obj).ykvSelf;  // ImageIcon
		}
		if( obj instanceof YKLookLabel )
		{
			obj = ((YKLookLabel) obj).ykmCheckClone();
			lab = (JLabel) ((YKLookLabel) obj).ykvSelf;  // JLabel
			if( ykvRender == null )
			{
				ykvRender = new YKLookListCellRendererJ();
				ykvSelf.setCellRenderer( ykvRender );
			}
			if( lab.isPreferredSizeSet() )
			{
				ykvRender.setPreferredSize( lab.getPreferredSize() );
			}
			lab.setOpaque( true );
			return lab;
		}
		return null;
	}
	public boolean yk_setItem( StringTokenizer prmStrTok )
	{
		int    len = Integer.parseInt( prmStrTok.nextToken() );
		int    idx;
		String tok;
		Object obj;

		ykvModel.clear();
		ykvItems.clear();

		for( idx = 0; idx < len; idx++ )
		{
			tok = prmStrTok.nextToken();
			obj = ykmDecodeItem( tok );
			if( obj != null )
			{
				ykvModel.addElement( obj );
				ykvItems.add( tok );
			}
		}
		return true;  // need scan for 'end'
	}
	public boolean yk_addItem( StringTokenizer prmStrTok )
	{
		int    pos = Integer.parseInt( prmStrTok.nextToken() );
		int    len = Integer.parseInt( prmStrTok.nextToken() );
		int    idx;
		String tok;
		Object obj;

		if( pos < 0 )
			pos = ykvItems.size();

		for( idx = 0; idx < len; idx++ )
		{
			tok = prmStrTok.nextToken();
			obj = ykmDecodeItem( tok );
			if( obj != null )
			{
				if( ykvItems.size() <= pos )
				{
					ykvModel.addElement( obj );
					ykvItems.add( tok );
				}
				else
				{
					ykvModel.add( pos, obj );
					ykvItems.add( pos, tok );
				}
				pos++;
			}
		}
		return true;  // need scan for 'end'
	}
	public boolean yk_replaceItem( StringTokenizer prmStrTok )
	{
		int    pos = Integer.parseInt( prmStrTok.nextToken() );
		String tok = prmStrTok.nextToken();
		Object obj = ykmDecodeItem( tok );

		if( ykvItems.size() < 1 )
			return true;  // need scan for 'end';

		if( pos < 0 )
			pos = 0;
		if( pos >= ykvItems.size() )
			pos = ykvItems.size() - 1;

		ykvModel.set( pos, obj );
		ykvItems.set( pos, obj );

		return true;  // need scan for 'end'
	}
	public boolean yk_removeAtItem( StringTokenizer prmStrTok )
	{
		String tok = prmStrTok.nextToken();
		int    idx = ykvItems.indexOf( tok );

		if( 0 <= idx && idx < ykvItems.size() )
		{
			ykvModel.remove( idx );
			ykvItems.remove( idx );
		}
		return true;  // need scan for 'end'
	}
	public boolean yk_removeAtIndex( StringTokenizer prmStrTok )
	{
		int idx = Integer.parseInt( prmStrTok.nextToken() );

		if( 0 <= idx && idx < ykvItems.size() )
		{
			ykvModel.remove( idx );
			ykvItems.remove( idx );
		}
		return true;  // need scan for 'end'
	}
	public boolean yk_scrollToIndex( StringTokenizer prmStrTok )
	{
		int idx = Integer.parseInt( prmStrTok.nextToken() );

		if( 0 <= idx && idx < ykvItems.size() )
			ykvSelf.ensureIndexIsVisible( idx );

		return true;  // need scan for 'end'
	}
	public boolean yk_scrollToItem( StringTokenizer prmStrTok )
	{
		int idx = ykvItems.indexOf( prmStrTok.nextToken() );

		if( 0 <= idx && idx < ykvItems.size() )
			ykvSelf.ensureIndexIsVisible( idx );

		return true;  // need scan for 'end'
	}
	public boolean yk_selectNone( StringTokenizer prmStrTok )
	{
		ykvSelf.clearSelection();
		return true;  // need scan for 'end'
	}
	void ykmSelectIndex( int[] prmArray, int prmMax )
	{
		int cnt;

		if( prmMax > 0 )
		{
			for( cnt = prmMax; cnt < prmArray.length; cnt++ )
			{
				prmArray[cnt] = prmArray[prmMax - 1];
			}
		}
		else
		{
			if( prmArray.length > 0 )
				prmArray = new int[0];
		}
		ykvSelf.setSelectedIndices( prmArray );
	}
	public boolean yk_setSelectedIndex( StringTokenizer prmStrTok )
	{
		int   len = Integer.parseInt( prmStrTok.nextToken() );
		int[] sel = new int[len];
		int   num = 0;
		int   cnt;
		int   idx;

		for( cnt = 0; cnt < len; cnt++ )
		{
			idx = Integer.parseInt( prmStrTok.nextToken() );
			if( 0 <= idx && idx < ykvItems.size() )
			{
				sel[num] = idx;
				num++;
			}
		}
		ykmSelectIndex( sel, num );
		return true;  // need scan for 'end'
	}
	public boolean yk_setSelectedItem( StringTokenizer prmStrTok )
	{
		int    len = Integer.parseInt( prmStrTok.nextToken() );
		int[]  sel = new int[len];
		int    num = 0;
		int    cnt;
		int    idx;

		for( cnt = 0; cnt < len; cnt++ )
		{
			idx = ykvItems.indexOf( prmStrTok.nextToken() );
			if( 0 <= idx && idx < ykvItems.size() )
			{
				sel[num] = idx;
				num++;
			}
		}
		ykmSelectIndex( sel, num );
		return true;  // need scan for 'end'
	}
	public boolean yk_setSelectMode( StringTokenizer prmStrTok )
	{
		int sm;

		switch( prmStrTok.nextToken().charAt(0) )
		{
			case 'S':
			case 's':
					sm = ListSelectionModel.SINGLE_SELECTION;
					break;
			case 'I':
			case 'i':
					sm = ListSelectionModel.SINGLE_INTERVAL_SELECTION;
					break;
			case 'M':
			case 'm':
			default:
					sm = ListSelectionModel.MULTIPLE_INTERVAL_SELECTION;
					break;
		}
		ykvSelf.setSelectionMode( sm );
		return true;  // need scan for 'end'
	}
	public boolean yk_setRow( StringTokenizer prmStrTok )
	{
		int cnt = Integer.parseInt( prmStrTok.nextToken() );

		ykvSelf.setVisibleRowCount( cnt );
		return true;  // need scan for 'end'
	}

	public boolean yk_setListener( StringTokenizer prmStrTok )
	{
		ykmSetCallback( prmStrTok );

		ykvClickCount = Integer.parseInt( prmStrTok.nextToken() );

		ykvSelf.removeMouseListener( this );

		if( ykvCallback != null )
			ykvSelf.addMouseListener( this );

		return true;  // need scan for 'end'
	}
	public void mouseClicked( MouseEvent prmEvent )
	{
		long  wh  = prmEvent.getWhen();
		int[] sel = ykvSelf.getSelectedIndices();

		if( (wh - ykvMouseWhen) < 1000 )
			return;
		if( prmEvent.getClickCount() != ykvClickCount )
			return;

		if( sel.length == 1 )
		{
			ykmSendEventListClick( prmEvent );
		}
		ykvMouseWhen = prmEvent.getWhen();
	}
	public void mousePressed ( MouseEvent prmEvent ) { }
	public void mouseReleased( MouseEvent prmEvent ) { }
	public void mouseEntered ( MouseEvent prmEvent ) { }
	public void mouseExited  ( MouseEvent prmEvent ) { }

	void ykmSendEventListClick( MouseEvent prmEvent )
	{
		String keys = ykmModifierKeys( prmEvent );
		String head = ykmMakeEventHeader( "YKLookEventListClick" );

		if( head == null )
			return;

		ykvCommon.ykmSendEventAny( head + keys + ":end:" );
	}
	public void ykmDumpGuiInfo( StringBuffer prmBuf )
			throws YKLookExceptionEncoding
	{
		String item;
		int    idx;
		int[]  sel = ykvSelf.getSelectedIndices();

		prmBuf.append( ykvId.toString() ).append( ":" ).append( sel.length );

		for( idx = 0; idx < sel.length; idx++ )
		{
			item = (String) ykvItems.get( sel[idx] );

			prmBuf.append(":").append( sel[idx] ).append(":").append( item );
		}
		prmBuf.append( ":end:" );
	}
}
class YKLookPopupList extends YKLookComponent implements ActionListener
{
	JComboBox               ykvSelf;
	YKLookListCellRendererJ ykvRender = null;
	ArrayList               ykvItems  = new ArrayList();

	YKLookPopupList()
	{
		ykvComponent = ykvSelf = new JComboBox();
	}

	Object ykmDecodeItem( String prmToken )
	{
		char   typ = prmToken.charAt(0);
		String dat = prmToken.substring(2);
		Object obj;
		JLabel lab;

		if( typ == 'S' )
		{
			return ykvCommon.ykmEncodingFrom(
						YKLookCommon.ykmDecodeFromHex( dat ) );
		}
		if( typ == 'i' || typ == 'd' )
		{
			return dat;
		}

		if( ykvSelf.isEditable() )
		{
			return "?";
		}

		obj = ykvCommon.ykmFindObjectFromId( dat );
		if( obj instanceof YKLookImage )
		{
			return ((YKLookImage) obj).ykvSelf;  // ImageIcon
		}
		if( obj instanceof YKLookLabel )
		{
			obj = ((YKLookLabel) obj).ykmCheckClone();
			lab = (JLabel) ((YKLookLabel) obj).ykvSelf;  // JLabel
			if( ykvRender == null )
			{
				ykvRender = new YKLookListCellRendererJ();
				ykvSelf.setRenderer( ykvRender );
			}
			if( lab.isPreferredSizeSet() )
			{
				ykvRender.setPreferredSize( lab.getPreferredSize() );
			}
			lab.setOpaque( true );
			return lab;
		}
		return null;
	}
	public boolean yk_setItem( StringTokenizer prmStrTok )
	{
		int    len = Integer.parseInt( prmStrTok.nextToken() );
		int    idx;
		String tok;
		Object obj;

		ykvSelf.removeAllItems();
		ykvItems.clear();

		for( idx = 0; idx < len; idx++ )
		{
			tok = prmStrTok.nextToken();
			obj = ykmDecodeItem( tok );
			if( obj != null )
			{
				ykvSelf.addItem( obj );
				ykvItems.add( tok );
			}
		}
		return true;  // need scan for 'end'
	}
	public boolean yk_addItem( StringTokenizer prmStrTok )
	{
		int    pos = Integer.parseInt( prmStrTok.nextToken() );
		int    len = Integer.parseInt( prmStrTok.nextToken() );
		int    idx;
		String tok;
		Object obj;

		if( pos < 0 )
			pos = ykvItems.size();

		for( idx = 0; idx < len; idx++ )
		{
			tok = prmStrTok.nextToken();
			obj = ykmDecodeItem( tok );
			if( obj != null )
			{
				if( ykvItems.size() <= pos )
				{
					ykvSelf.addItem( obj );
					ykvItems.add( tok );
				}
				else
				{
					ykvSelf.insertItemAt( obj, pos );
					ykvItems.add( pos, tok );
				}
				pos++;
			}
		}
		return true;  // need scan for 'end'
	}
	public boolean yk_removeAtItem( StringTokenizer prmStrTok )
	{
		String tok = prmStrTok.nextToken();
		int    idx = ykvItems.indexOf( tok );

		if( 0 <= idx && idx < ykvItems.size() )
		{
			ykvSelf.removeItemAt( idx );
			ykvItems.remove( idx );
		}
		return true;  // need scan for 'end'
	}
	public boolean yk_removeAtIndex( StringTokenizer prmStrTok )
	{
		int idx = Integer.parseInt( prmStrTok.nextToken() );

		if( 0 <= idx && idx < ykvItems.size() )
		{
			ykvSelf.removeItemAt( idx );
			ykvItems.remove( idx );
		}
		return true;  // need scan for 'end'
	}
	public boolean yk_setSelectedIndex( StringTokenizer prmStrTok )
	{
		int idx = Integer.parseInt( prmStrTok.nextToken() );

		ykvSelf.setSelectedIndex( idx );
		return true;  // need scan for 'end'
	}
	public boolean yk_setSelectedItem( StringTokenizer prmStrTok )
	{
		String tok = prmStrTok.nextToken();
		int    idx = ykvItems.indexOf( tok );

		if( 0 <= idx && idx < ykvItems.size() )
		{
			ykvSelf.setSelectedIndex( idx );
		}
		else
		{
			tok = ykvCommon.ykmEncodingFrom(
					YKLookCommon.ykmDecodeFromHex( tok.substring(2) ) );
			ykvSelf.setSelectedItem( tok );
		}
		return true;  // need scan for 'end'
	}
	public boolean yk_setRow( StringTokenizer prmStrTok )
	{
		int cnt = Integer.parseInt( prmStrTok.nextToken() );

		ykvSelf.setMaximumRowCount( cnt );
		return true;  // need scan for 'end'
	}
	public boolean yk_setListener( StringTokenizer prmStrTok )
	{
		ykmSetCallback( prmStrTok );

		ykvSelf.removeActionListener( this );

		if( ykvCallback != null )
			ykvSelf.addActionListener( this );

		return true;  // need scan for 'end'
	}

	public void actionPerformed( ActionEvent prmEvent )
	{
		ykmSendEventAction( prmEvent );
	}

	public void ykmDumpGuiInfo( StringBuffer prmBuf )
			throws YKLookExceptionEncoding
	{
		String item;
		int    idx = ykvSelf.getSelectedIndex();

		if( idx < 0 )
		{
			item = "S/" + ykvCommon.ykmDumpString(
									(String) ykvSelf.getSelectedItem() );
		}
		else
		{
			item = (String) ykvItems.get( idx );
		}
		prmBuf.append( ykvId.toString() +":" + idx + ":"+ item + ":end:" );
	}
}
class YKLookComboBox extends YKLookPopupList
{
	YKLookComboBox()
	{
		ykvSelf.setEditable( true );
	}
}
class YKLookTextComponent extends YKLookScrollable
{
	JTextComponent ykvSelf;

	public boolean yk_cut( StringTokenizer prmStrTok )
	{
		ykvSelf.cut();
		return true;  // need scan for 'end'
	}
	public boolean yk_copy( StringTokenizer prmStrTok )
	{
		ykvSelf.copy();
		return true;  // need scan for 'end'
	}
	public boolean yk_paste( StringTokenizer prmStrTok )
	{
		ykvSelf.paste();
		return true;  // need scan for 'end'
	}
	public boolean yk_setCaretPosition( StringTokenizer prmStrTok )
	{
		int pos = Integer.parseInt( prmStrTok.nextToken() );

		if( pos < 0 )
		{
			pos = ykvSelf.getText().length();
		}
		ykvSelf.setCaretPosition( pos );
		return true;  // need scan for 'end'
	}
	public boolean yk_setEditable( StringTokenizer prmStrTok )
	{
		char tf = prmStrTok.nextToken().charAt(0);

		ykvSelf.setEditable( tf == 'T' || tf == 't' );
		return true;  // need scan for 'end'
	}
	public boolean yk_setText( StringTokenizer prmStrTok )
	{
		String txt = ykvCommon.ykmNextString( prmStrTok );

		ykvSelf.setText( txt );
		return true;  // need scan for 'end'
	}
	public boolean yk_replaceSelection( StringTokenizer prmStrTok )
	{
		String txt = ykvCommon.ykmNextString( prmStrTok );

		ykvSelf.replaceSelection( txt );
		return true;  // need scan for 'end'
	}
	public boolean yk_selectAll( StringTokenizer prmStrTok )
	{
		ykvSelf.selectAll();
		return true;  // need scan for 'end'
	}

	public void ykmDumpGuiInfo( StringBuffer prmBuf )
			throws YKLookExceptionEncoding
	{
		String tx = ykvSelf.getText();
		int    ss = ykvSelf.getSelectionStart();
		int    se = ykvSelf.getSelectionEnd();
		String s1 = tx.substring(  0, ss );
		String s2 = tx.substring( ss, se );
		String s3 = tx.substring( se );

		tx = ykvCommon.ykmDumpString( tx );
		s1 = ykvCommon.ykmDumpString( s1 );
		s2 = ykvCommon.ykmDumpString( s2 );
		s3 = ykvCommon.ykmDumpString( s3 );
		prmBuf.append( ykvId.toString() + ":" +
						tx + ":" + s1 + ":" + s2 + ":" + s3 + ":end:" );
	}
}
class YKLookPasswordField extends YKLookTextComponent
{
	YKLookPasswordField()
	{
		ykvComponent = ykvSelf = new JPasswordField();
		ykvSelf.setEditable( true );
	}

	public boolean yk_setColumns( StringTokenizer prmStrTok )
	{
		int col = Integer.parseInt( prmStrTok.nextToken() );

		((JPasswordField) ykvSelf).setColumns( col );
		return true;  // need scan for 'end'
	}
	public boolean yk_setEchoChar( StringTokenizer prmStrTok )
	{
		String s = ykvCommon.ykmNextString( prmStrTok );
		char[] c;

		if( s == null )
		{
			((JPasswordField) ykvSelf).setEchoChar( (char) 0 );
		}
		else
		{
			c = s.toCharArray();
			if( c.length == 1 )
			{
				((JPasswordField) ykvSelf).setEchoChar( c[0] );
			}
		}
		return true;  // need scan for 'end'
	}

	public void ykmDumpGuiInfo( StringBuffer prmBuf )
			throws YKLookExceptionEncoding
	{
		char[] c = ((JPasswordField) ykvSelf).getPassword();
		String s;
		int    i;

		try
		{
			s = ykvCommon.ykmDumpPassword( c );
			prmBuf.append( ykvId.toString() + ":" +
							s + ":empty:empty:empty:end:" );
		}
		finally
		{
			for( i = 0; i < c.length; i++ )
			{
				c[i] = 0xffff;
			}
		}
	}
}
class YKLookTextField extends YKLookTextComponent
{
	YKLookTextField()
	{
		ykvComponent = ykvSelf = new JTextField();
		ykvSelf.setEditable( true );
	}

	public boolean yk_setColumns( StringTokenizer prmStrTok )
	{
		int col = Integer.parseInt( prmStrTok.nextToken() );

		((JTextField) ykvSelf).setColumns( col );
		return true;  // need scan for 'end'
	}
	public boolean yk_setHorizontalAlignment( StringTokenizer prmStrTok )
			throws YKLookException
	{
		((JTextField) ykvSelf).setHorizontalAlignment(
												ykmHorizontal( prmStrTok ) );
		return true;  // need scan for 'end'
	}
}
class YKLookTextArea extends YKLookTextComponent
{
	YKLookTextArea()
	{
		ykvComponent = ykvSelf = new JTextArea();
		ykvSelf.setEditable( true );
		((JTextArea) ykvSelf).setLineWrap( true );
	}
	public boolean yk_setRows( StringTokenizer prmStrTok )
	{
		int row = Integer.parseInt( prmStrTok.nextToken() );

		((JTextArea) ykvSelf).setRows( row );
		return true;  // need scan for 'end'
	}
	public boolean yk_setColumns( StringTokenizer prmStrTok )
	{
		int col = Integer.parseInt( prmStrTok.nextToken() );

		((JTextArea) ykvSelf).setColumns( col );
		return true;  // need scan for 'end'
	}
	public boolean yk_append( StringTokenizer prmStrTok )
	{
		String txt = ykvCommon.ykmNextString( prmStrTok );

		((JTextArea) ykvSelf).append( txt );
		return true;  // need scan for 'end'
	}
	public boolean yk_setLineWrap( StringTokenizer prmStrTok )
	{
		char ch = prmStrTok.nextToken().charAt(0);

		((JTextArea) ykvSelf).setLineWrap( ( ch == 't' || ch == 'T' ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_setTabSize( StringTokenizer prmStrTok )
	{
		int ts = Integer.parseInt( prmStrTok.nextToken() );

		((JTextArea) ykvSelf).setTabSize( ts );
		return true;  // need scan for 'end'
	}
	public boolean yk_setWrapStyleWord( StringTokenizer prmStrTok )
	{
		char ch = prmStrTok.nextToken().charAt(0);

		((JTextArea) ykvSelf).setWrapStyleWord( ( ch == 't' || ch == 'T' ) );
		return true;  // need scan for 'end'
	}
}
class YKLookCursor extends YKLookSwingBase
{
	Cursor ykvSelf;

	public boolean yk_setImage( StringTokenizer prmStrTok )
	{
		String      tok = prmStrTok.nextToken();
		int         x   = Integer.parseInt( prmStrTok.nextToken() );
		int         y   = Integer.parseInt( prmStrTok.nextToken() );
		String      nam = ykvCommon.ykmNextString( prmStrTok );
		Point       poi = new Point( x, y );
		Toolkit     kit = Toolkit.getDefaultToolkit();
		YKLookImage img;

		img = (YKLookImage) ykvCommon.ykmFindObjectFromId( tok );
		if( img == null )
		{
			ykvSelf = Cursor.getDefaultCursor();
			return true;  // need scan for 'end'
		}
		if( nam == null )
			nam = "user custom cursor";

		ykvSelf = kit.createCustomCursor( img.ykvSelf.getImage(), poi, nam );
		return true;  // need scan for 'end'
	}
	public boolean yk_setPred( StringTokenizer prmStrTok )
	{
		ykvSelf = ykmGetCursor( prmStrTok.nextToken() );
		return true;  // need scan for 'end'
	}
}
class YKLookImage extends YKLookSwingBase
{
	ImageIcon ykvSelf;

	public boolean yk_setRawData( StringTokenizer prmStrTok )
			throws YKLookException
	{
		String  hex = prmStrTok.nextToken();
		byte[]  bin = YKLookCommon.ykmDecodeFromHex( hex );

		ykvSelf = new ImageIcon( bin );
		return true;  // need scan for 'end'
	}
	public boolean yk_setUrl( StringTokenizer prmStrTok )
			throws java.net.MalformedURLException
	{
		String  url  = ykvCommon.ykmNextString( prmStrTok );
		String  file = ykvCommon.ykmNextString( prmStrTok );
		Image   img  = ykvCommon.ykmGetImageFromUrl( url, file );

		ykvSelf = new ImageIcon( img );
		//ykvSelf = new ImageIcon( ykvCommon.ykmGetUrl( url, file ) );
		return true;  // need scan for 'end'
	}
}
class YKLookColor extends YKLookSwingBase
{
	Color ykvSelf = null;

	public boolean yk_setColor( StringTokenizer prmStrTok )
	{
		ykvSelf = ykvCommon.ykmDecodeColor( prmStrTok.nextToken() );
		return true;  // need scan for 'end'
	}
}
class YKLookFont extends YKLookSwingBase
{
	static final String[] ykvJavaFontNames = { "Dialog", "DialogInput",
										"MonoSpaced", "Serif", "SansSerif" };
	static String[] ykvLocalFontNames = null;

	Font ykvSelf;

	public boolean yk_setFont( StringTokenizer prmStrTok )
	{
		String nam, tok;
		int    idx, sty, siz;

		for( ;; )
		{
			tok = prmStrTok.nextToken();
			if( tok.equalsIgnoreCase("end") )
			{
				ykvSelf = new Font( null, Font.PLAIN, 12 );
				return false;  // scan the next command immediately
			}
			nam = ykvCommon.ykmEncodingFrom(
									YKLookCommon.ykmDecodeFromHex( tok ) );
			if( nam == null )
				break;
			
			for( idx = 0; idx < ykvJavaFontNames.length; idx++ )
			{
				if( nam.equalsIgnoreCase( ykvJavaFontNames[idx] ) )
				{
					nam = ykvJavaFontNames[idx];
					break;
				}
			}
			if( idx < ykvJavaFontNames.length )
				break;

			if( ykvLocalFontNames == null )
				ykvLocalFontNames = ykmMakeLocalFontNames();

			for( idx = 0; idx < ykvLocalFontNames.length &&
								ykvLocalFontNames[idx] != null; idx++ )
			{
				if( nam.equalsIgnoreCase( ykvLocalFontNames[idx] ) )
				{
					nam = ykvLocalFontNames[idx];
					break;
				}
			}
			if( idx < ykvLocalFontNames.length &&
						ykvLocalFontNames[idx] != null )
				break;

			while( ! prmStrTok.nextToken().equals("yk_style_end") )
				;
			prmStrTok.nextToken();  // size
		}
		sty = 0;
		for( ;; )
		{
			tok = prmStrTok.nextToken();
			if( tok.equals("yk_style_end") )
				break;

			switch( tok.charAt(0) )
			{
				case 'P':
				case 'p': sty += Font.PLAIN; break;
				case 'B':
				case 'b': sty += Font.BOLD; break;
				case 'I':
				case 'i': sty += Font.ITALIC; break;
				default: // nothing
						break;
			}
		}
		siz = Integer.parseInt( prmStrTok.nextToken() );
		ykvSelf = new Font( nam, sty, siz );
		return true;  // need scan for 'end'
	}
	String[] ykmMakeLocalFontNames()
	{
		int      cnt, idx;
		String   preFamily, family, face, lgn;
		String[] fontNames;
		Font[]   fonts = GraphicsEnvironment.
								getLocalGraphicsEnvironment().
								getAllFonts();

		fontNames = new String[ fonts.length * 3 ];
		preFamily = "";
		cnt       = 0;

		for( idx = 0; idx < fonts.length; idx++ )
		{
			family = fonts[idx].getFamily();
			if( ! family.equals( preFamily ) )
			{
				preFamily = family;
				fontNames[cnt] = family;
				cnt++;
			}
			face = fonts[idx].getFontName();
			if( ! face.equals( family ) )
			{
				fontNames[cnt] = face;
				cnt++;
			}
			lgn = fonts[idx].getName();
			if( ! ( lgn.equals( face ) || lgn.equals( family ) ) )
			{
				fontNames[cnt] = lgn;
				cnt++;
			}
		}
		if( cnt < fontNames.length )
			fontNames[cnt] = null;

		return fontNames;
	}
}
class YKLookPanel extends YKLookComponent
{
	JPanel        ykvSelf;
	LayoutManager ykvLayMan;

	YKLookPanel()
	{
		ykvComponent = ykvSelf = new JPanel();
	}

	void ykmSetMainPanel( JPanel prmPanel )
	{
		ykvComponent = ykvSelf = prmPanel;
		ykvLayMan    = prmPanel.getLayout();
	}

	public boolean yk_setLayout( StringTokenizer prmStrTok )
	{
		ykvSelf.setLayout( ykvLayMan );
		return true;  // need scan for 'end'
	}

	// id
	public boolean yk_add( StringTokenizer prmStrTok )
	{
		String        tok = prmStrTok.nextToken();
		YKLookElement elm;

		elm = (YKLookElement) ykvCommon.ykmFindObjectFromId( tok );
		elm = elm.ykmCheckClone();
		if( elm != null )
		{
			ykvSelf.add( elm.ykmComponent() );
		}
		return true;  // need scan for 'end'
	}
}
class YKLookPanelFlow extends YKLookPanel
{
	YKLookPanelFlow()
	{
		ykvLayMan = ykvSelf.getLayout();
	}

	// hgap
	public boolean yk_setHgap( StringTokenizer prmStrTok )
	{
		int hgap = Integer.parseInt( prmStrTok.nextToken() );

		((FlowLayout)ykvLayMan).setHgap( hgap );
		return true;  // need scan for 'end'
	}
	// vgap
	public boolean yk_setVgap( StringTokenizer prmStrTok )
	{
		int vgap = Integer.parseInt( prmStrTok.nextToken() );

		((FlowLayout)ykvLayMan).setVgap( vgap );
		return true;  // need scan for 'end'
	}

	public boolean yk_setAlignment( StringTokenizer prmStrTok )
	{
		String tok = prmStrTok.nextToken();
		int jus;

		tok = tok.substring(0,3).toLowerCase();
		for( ;; )
		{
			if( tok.equals("lef") ) { jus = FlowLayout.LEFT    ; break; }
			if( tok.equals("rig") ) { jus = FlowLayout.RIGHT   ; break; }
			if( tok.equals("lea") ) { jus = FlowLayout.LEADING ; break; }
			if( tok.equals("tra") ) { jus = FlowLayout.TRAILING; break; }

			jus = FlowLayout.CENTER;  // default
			break;
		}
		((FlowLayout) ykvLayMan).setAlignment( jus );
		return true;  // need scan for 'end'
	}
}
class YKLookPanelBox extends YKLookPanel
{
	public boolean yk_setAxis( StringTokenizer prmStrTok )
	{
		String tok = prmStrTok.nextToken();
		int    axis;

		switch( tok.charAt(0) )
		{
			case 'v':
			case 'V':
			case 'y':
			case 'Y':
					axis = BoxLayout.Y_AXIS; break;
			default:
					axis = BoxLayout.X_AXIS; break;
		}
		ykvLayMan = new BoxLayout( ykvSelf, axis );
		return true;  // need scan for 'end'
	}
}
class YKLookPanelGrid extends YKLookPanel
{
	public boolean yk_setGrid( StringTokenizer prmStrTok )
	{
		int row = Integer.parseInt( prmStrTok.nextToken() );
		int col = Integer.parseInt( prmStrTok.nextToken() );

		ykvLayMan = new GridLayout( row, col );
		return true;  // need scan for 'end'
	}

	// hgap
	public boolean yk_setHgap( StringTokenizer prmStrTok )
	{
		int hgap = Integer.parseInt( prmStrTok.nextToken() );

		((GridLayout)ykvLayMan).setHgap( hgap );
		return true;  // need scan for 'end'
	}
	// vgap
	public boolean yk_setVgap( StringTokenizer prmStrTok )
	{
		int vgap = Integer.parseInt( prmStrTok.nextToken() );

		((GridLayout)ykvLayMan).setVgap( vgap );
		return true;  // need scan for 'end'
	}
}
class YKLookPanelGridBag extends YKLookPanel
{
	YKLookPanelGridBag()
	{
		ykvLayMan = new GridBagLayout();
	}
	public boolean yk_add( StringTokenizer prmStrTok )
	{
		String             tok = prmStrTok.nextToken();
		YKLookElement      elm;
		GridBagConstraints gbc = new GridBagConstraints();

		elm = (YKLookElement) ykvCommon.ykmFindObjectFromId( tok );
		elm = elm.ykmCheckClone();
		if( elm == null )
			return true;  // need scan for 'end'

		tok = prmStrTok.nextToken();
		if( tok.equalsIgnoreCase( "REL" ) )
			gbc.gridx = GridBagConstraints.RELATIVE;
		else
			gbc.gridx = Integer.parseInt( tok );

		tok = prmStrTok.nextToken();
		if( tok.equalsIgnoreCase( "REL" ) )
			gbc.gridy = GridBagConstraints.RELATIVE;
		else
			gbc.gridy = Integer.parseInt( tok );

		tok = prmStrTok.nextToken();
		if( tok.equalsIgnoreCase( "RMD" ) )
			gbc.gridwidth = GridBagConstraints.REMAINDER;
		else
			gbc.gridwidth = Integer.parseInt( tok );

		tok = prmStrTok.nextToken();
		if( tok.equalsIgnoreCase( "RMD" ) )
			gbc.gridheight = GridBagConstraints.REMAINDER;
		else
			gbc.gridheight = Integer.parseInt( tok );

		tok = prmStrTok.nextToken();
		switch( tok.charAt(0) )
		{
			case 'H':
			case 'h': gbc.fill = GridBagConstraints.HORIZONTAL; break;
			case 'V':
			case 'v': gbc.fill = GridBagConstraints.VERTICAL; break;
			case 'B':
			case 'b': gbc.fill = GridBagConstraints.BOTH; break;
			case 'N':
			case 'n':
			default:  gbc.fill = GridBagConstraints.NONE; break;
		}
		gbc.insets.top    = Integer.parseInt( prmStrTok.nextToken() );
		gbc.insets.left   = Integer.parseInt( prmStrTok.nextToken() );
		gbc.insets.bottom = Integer.parseInt( prmStrTok.nextToken() );
		gbc.insets.right  = Integer.parseInt( prmStrTok.nextToken() );
		gbc.ipadx         = Integer.parseInt( prmStrTok.nextToken() );
		gbc.ipady         = Integer.parseInt( prmStrTok.nextToken() );

		tok = prmStrTok.nextToken();
		if( tok.equalsIgnoreCase( "DEF" ) )
		{
			//if( gbc.fill == GridBagConstraints.HORIZONTAL ||
			//	gbc.fill == GridBagConstraints.BOTH )
			//	gbc.weightx = 100.0;
			//else
				gbc.weightx = 0.0;
		}
		else
		{
			gbc.weightx = Double.parseDouble( tok );
		}
		tok = prmStrTok.nextToken();
		if( tok.equalsIgnoreCase( "DEF" ) )
		{
			//if( gbc.fill == GridBagConstraints.VERTICAL ||
			//	gbc.fill == GridBagConstraints.BOTH )
			//	gbc.weighty = 100.0;
			//else
				gbc.weighty = 0.0;
		}
		else
		{
			gbc.weighty = Double.parseDouble( tok );
		}
		tok = prmStrTok.nextToken().toLowerCase();
		for( ;; )
		{
			if( tok.equals("n") )
			{ gbc.anchor = GridBagConstraints.NORTH; break; }
			if( tok.equals("ne") )
			{ gbc.anchor = GridBagConstraints.NORTHEAST; break; }
			if( tok.equals("nw") )
			{ gbc.anchor = GridBagConstraints.NORTHWEST; break; }
			if( tok.equals("s") )
			{ gbc.anchor = GridBagConstraints.SOUTH; break; }
			if( tok.equals("se") )
			{ gbc.anchor = GridBagConstraints.SOUTHEAST; break; }
			if( tok.equals("sw") )
			{ gbc.anchor = GridBagConstraints.SOUTHWEST; break; }
			if( tok.equals("e") )
			{ gbc.anchor = GridBagConstraints.EAST; break; }
			if( tok.equals("w") )
			{ gbc.anchor = GridBagConstraints.WEST; break; }

			gbc.anchor = GridBagConstraints.CENTER;  // default
			break;
		}
		ykvSelf.add( elm.ykmComponent(), gbc );
		return true;  // need scan for 'end'
	}
}
class YKLookPanelCard extends YKLookPanel
{
	YKLookPanelCard()
	{
		ykvLayMan = new CardLayout();
	}

	// hgap
	public boolean yk_setHgap( StringTokenizer prmStrTok )
	{
		int hgap = Integer.parseInt( prmStrTok.nextToken() );

		((CardLayout)ykvLayMan).setHgap( hgap );
		return true;  // need scan for 'end'
	}
	// vgap
	public boolean yk_setVgap( StringTokenizer prmStrTok )
	{
		int vgap = Integer.parseInt( prmStrTok.nextToken() );

		((CardLayout)ykvLayMan).setVgap( vgap );
		return true;  // need scan for 'end'
	}

	public boolean yk_add( StringTokenizer prmStrTok )
	{
		String        tok = prmStrTok.nextToken();
		String        key = prmStrTok.nextToken();
		YKLookElement elm;

		elm = (YKLookElement) ykvCommon.ykmFindObjectFromId( tok );
		elm = elm.ykmCheckClone();
		if( elm != null )
		{
			ykvSelf.add( elm.ykmComponent(), key );
		}
		return true;  // need scan for 'end'
	}
	public boolean yk_show( StringTokenizer prmStrTok )
	{
		String key = prmStrTok.nextToken();

		((CardLayout)ykvLayMan).show( ykvSelf, key );
		return true;  // need scan for 'end'
	}
	public boolean yk_first( StringTokenizer prmStrTok )
	{
		((CardLayout)ykvLayMan).first( ykvSelf );
		return true;  // need scan for 'end'
	}
	public boolean yk_last( StringTokenizer prmStrTok )
	{
		((CardLayout)ykvLayMan).last( ykvSelf );
		return true;  // need scan for 'end'
	}
	public boolean yk_next( StringTokenizer prmStrTok )
	{
		((CardLayout)ykvLayMan).next( ykvSelf );
		return true;  // need scan for 'end'
	}
	public boolean yk_previous( StringTokenizer prmStrTok )
	{
		((CardLayout)ykvLayMan).previous( ykvSelf );
		return true;  // need scan for 'end'
	}
}
class YKLookPanelQuint extends YKLookPanel
{
	YKLookPanelQuint()
	{
		ykvLayMan = new BorderLayout();
	}

	// hgap
	public boolean yk_setHgap( StringTokenizer prmStrTok )
	{
		int hgap = Integer.parseInt( prmStrTok.nextToken() );

		((BorderLayout)ykvLayMan).setHgap( hgap );
		return true;  // need scan for 'end'
	}
	// vgap
	public boolean yk_setVgap( StringTokenizer prmStrTok )
	{
		int vgap = Integer.parseInt( prmStrTok.nextToken() );

		((BorderLayout)ykvLayMan).setVgap( vgap );
		return true;  // need scan for 'end'
	}

	// id,pos
	public boolean yk_add( StringTokenizer prmStrTok )
	{
		String        tok = prmStrTok.nextToken();
		String        pos = prmStrTok.nextToken();
		YKLookElement elm;
		Container     rp;  // when root panel

		elm = (YKLookElement) ykvCommon.ykmFindObjectFromId( tok );
		elm = elm.ykmCheckClone();
		if( elm == null )
			return true;  // need scan for 'end'

		switch( pos.charAt(0) )
		{
			case 't': case 'T':
			case 'n': case 'N': pos = BorderLayout.NORTH; break;
			case 'b': case 'B':
			case 's': case 'S': pos = BorderLayout.SOUTH; break;
			case 'r': case 'R':
			case 'e': case 'E': pos = BorderLayout.EAST; break;
			case 'l': case 'L':
			case 'w': case 'W': pos = BorderLayout.WEST; break;
			case 'c': case 'C':
			default :           pos = BorderLayout.CENTER; break;
		}
		ykvSelf.add( elm.ykmComponent(), pos );
		return true;  // need scan for 'end'
	}
}
/* ------------------------------------------------------------------
class YKLookBox extends YKLookComponent  // not Panel
{
	Box ykvSelf;

	YKLookBox()
	{
		ykvComponent = new JPanel();
	}
	public boolean yk_createVerticalBox( StringTokenizer prmStrTok )
	{
		ykvSelf = Box.createVerticalBox();
		((JPanel) ykvComponent).add( ykvSelf );
		return true;  // need scan for 'end'
	}
	public boolean yk_createHorizontalBox( StringTokenizer prmStrTok )
	{
		ykvSelf = Box.createHorizontalBox();
		((JPanel) ykvComponent).add( ykvSelf );
		return true;  // need scan for 'end'
	}
	public boolean yk_setBorder( StringTokenizer prmStrTok )
	{
		String       tok = prmStrTok.nextToken();
		YKLookBorder bdr;

		bdr = (YKLookBorder) ykvCommon.ykmFindObjectFromId( tok );
		((JPanel) ykvComponent).setBorder( bdr.ykvSelf );
		return true;  // need scan for 'end'
	}

	// id
	public boolean yk_add( StringTokenizer prmStrTok )
	{
		String        tok = prmStrTok.nextToken();
		YKLookElement elm;

		elm = (YKLookElement) ykvCommon.ykmFindObjectFromId( tok );
		elm = elm.ykmCheckClone();
		if( elm != null )
		{
			ykvSelf.add( elm.ykmComponent() );
		}
		return true;  // need scan for 'end'
	}
}
------------------------------------------------------------------ */
class YKLookBoxFiller extends YKLookElement
{
	Component ykvComponent = null;

	YKLookElement ykmCheckClone()
	{
		YKLookBoxFiller bf;
		Dimension       min;
		Dimension       pre;
		Dimension       max;

		if( super.ykmCheckClone() == this )
			return this;

		bf  = new YKLookBoxFiller();

		min = ykvComponent.getMinimumSize();
		pre = ykvComponent.getPreferredSize();
		max = ykvComponent.getMaximumSize();

		bf.ykvComponent = new Box.Filler( min, pre, max );
		return bf;
	}
	Component ykmComponent()  // override
	{
		return ykvComponent;
	}

	public boolean yk_createGlue( StringTokenizer prmStrTok )
	{
		ykvComponent = Box.createGlue();
		return true;  // need scan for 'end'
	}
	public boolean yk_createVerticalGlue( StringTokenizer prmStrTok )
	{
		ykvComponent = Box.createVerticalGlue();
		return true;  // need scan for 'end'
	}
	public boolean yk_createHorizontalGlue( StringTokenizer prmStrTok )
	{
		ykvComponent = Box.createHorizontalGlue();
		return true;  // need scan for 'end'
	}

/* ----------------------------------------------------------------------
	public boolean yk_createVerticalStrut( StringTokenizer prmStrTok )
	{
		int pix = Integer.parseInt( prmStrTok.nextToken() );

		ykvComponent = Box.createVerticalStrut( pix );
		return true;  // need scan for 'end'
	}
	public boolean yk_createHorizontalStrut( StringTokenizer prmStrTok )
	{
		int pix = Integer.parseInt( prmStrTok.nextToken() );

		ykvComponent = Box.createHorizontalStrut( pix );
		return true;  // need scan for 'end'
	}
---------------------------------------------------------------------- */

	public boolean yk_createRigidArea( StringTokenizer prmStrTok )
	{
		ykvComponent = Box.createRigidArea( ykmDimension( prmStrTok ) );
		return true;  // need scan for 'end'
	}
	public boolean yk_createBoxFiller( StringTokenizer prmStrTok )
	{
		Dimension min = ykmDimension( prmStrTok );
		Dimension pre = ykmDimension( prmStrTok );
		Dimension max = ykmDimension( prmStrTok );

		ykvComponent = new Box.Filler( min, pre, max );
		return true;  // need scan for 'end'
	}
}

//
// end of program
//
