//
//  YKLookEncodingBase
//  YKLookEncoding_SJIS_Mac_KT
//
//  Copyright (c) 2009-2010 YorozuKotowariYa Ltd.
//
//
package jp.co.yorozukotowariya.yklook;


class YKLookEncodingBase
{
	String ykmEncodingFrom( byte[] prmData )
	{
		return null;
	}
	byte[] ykmEncodingTo( String prmStr ) throws YKLookException
	{
		return null;
	}
}

class YKLookEncoding_SJIS_Mac_KT extends YKLookEncodingBase
{
	String ykmEncodingFrom( byte[] prmData )
	{
		int [][] tbl = ykmGetTableFrom();
		char[]   uni = new char[ prmData.length * 4 ];
		int      cnt = 0;
		int      idx;
		int      wk1, wk2;
		char     ch;

		for( idx = 0; idx < prmData.length; idx++ )
		{
			wk1 = prmData[idx] & 0x00ff;
			switch( wk1 )
			{
				case 0x0011: ch = 0x2318; break;  // command key mark
				case 0x0012: ch = 0x2713; break;  // check mark
				case 0x0013: ch = 0x25c6; break;  // hishigata (SJIS=819f)
				case 0x0014: ch = 0xf8ff; break;  // apple mark (private area)
				case 0x005c: ch = 0x00a5; break;  // yen
				case 0x0080: ch = 0x005c; break;  // back slash
				case 0x00a0: ch = 0x00a0; break;  // NBSP
				case 0x00fd: ch = 0x00a9; break;  // C copyright mark
				case 0x00fe: ch = 0x2122; break;  // TM trade mark
				case 0x00ff: ch = 0x2026; break;  // ... (SJIS=8163)
				default:
					if( wk1 < 0x007f )  // single byte character
					{
						ch = (char) wk1;
						break;
					}
					if( 0x0081 <= wk1 && wk1 <= 0x009f )  // SJIS first byte
					{
						idx++;
						if( idx >= prmData.length )
						{
							ch = '?';
							break;
						}
						wk2 = prmData[idx] & 0x00ff;        // SJIS second byte
						if( wk2 < 0x0040 || 0x00fc < wk2 )
						{
							ch = '?';
							break;
						}
						ch = (char) tbl[wk1 - 0x0081][wk2 - 0x0040];
						if( 1 <= ch && ch <= 14 )              // KT7 spacial
						{
							wk1 = ch - 1;
							for( wk2 = 0; wk2 < 4; wk2++ )
							{
								ch = (char) ykvMac7tbl[wk1][wk2];
								if( ch == 0 )
									break;
								uni[cnt] = ch;
								cnt++;
							}
							ch = (char) ykvMac7tbl[wk1][wk2-1];
							cnt--;
						}
						break;
					}
					if( 0x00a1 <= wk1 && wk1 <= 0x00df )  // half-width kana
					{
						ch = (char) (0x00ff00 | (wk1 - 0x0040));
						break;
					}
					if( 0x00e0 <= wk1 && wk1 <= 0x00ef )  // SJIS first byte
					{
						idx++;
						if( idx >= prmData.length )
						{
							ch = '?';
							break;
						}
						wk2 = prmData[idx] & 0x00ff;        // SJIS second byte
						if( wk2 < 0x0040 || 0x00fc < wk2 )
						{
							ch = '?';
							break;
						}
						ch = (char) tbl[wk1 - 0x00c1][wk2 - 0x0040];
						if( 1 <= ch && ch <= 14 )              // KT7 spacial
						{
							wk1 = ch - 1;
							for( wk2 = 0; wk2 < 4; wk2++ )
							{
								ch = (char) ykvMac7tbl[wk1][wk2];
								if( ch == 0 )
									break;
								uni[cnt] = ch;
								cnt++;
							}
							ch = (char) ykvMac7tbl[wk1][wk2-1];
							cnt--;
						}
						break;
					}
					if( 0x00f0 <= wk1 && wk1 <= 0x00fc )  // SJIS first byte
					{                                     // (private area)
						idx++;
						ch = '?';
						break;
					}
					ch = '?';
					break;
			}
			if( ch == 0 )
				ch = '?';

			uni[cnt] = ch;
			cnt++;
		}
		return new String( uni, 0, cnt );
	}
	byte[] ykmEncodingTo( String prmStr ) throws YKLookException
	{
		int [][] tbl = ykmGetTableTo();
		char[]   org = prmStr.toCharArray();
		byte[]   dat = new byte[ org.length * 2 ];
		int      cnt = 0;
		byte[]   res;
		int      num, idx, hi, lo, wk1, wk2;

		for( num = 0; num < org.length; num++ )
		{
			if( org[num] > 0x0000ffff )
				throw new YKLookException("overBMP");

			hi = org[num] >>> 8;
			lo = org[num] & 0x00ff;

			switch( hi )
			{
				case   0: idx = 0; break;              // 0x00
				case   3: idx = 1; break;              // 0x03      
				case   4: idx = 2; break;              // 0x04
				case  32: case 33: case 34: case 35: case 36: case 37: case 38:  
				case  39: idx = hi - 29; break;        // 0x20 - 0x27
				case  48: idx = 11; break;             // 0x30
				case  50: idx = 12; break;             // 0x32
				case  51: idx = 13; break;             // 0x33
				case 255: idx = 14; break;             // 0xff
				default:
					if( 78 <= hi && hi <= 159 )        // 0x4e - 0x9f
					{
						idx = hi - 63;
					}
					else
					{
						throw new YKLookException("unknownHi");
						//idx = 0;
						//lo  = '?';
					}
					break;
			}
			wk1 = tbl[idx][lo] >>> 8;
			wk2 = tbl[idx][lo] & 0x00ff;
			if( wk1 == 0 )
			{
				if( wk2 == 0 )
					throw new YKLookException("unknownLo");
					//wk2 = '?';
			}
			else
			{
				dat[cnt] = (byte) wk1;
				cnt++;
			}
			dat[cnt] = (byte) wk2;
			cnt++;
		}
		res = new byte[ cnt ];
		System.arraycopy( dat, 0, res, 0, cnt );
		return res;
	}
	int[][] ykmGetTableFrom()
	{
		return null;
	}
	int[][] ykmGetTableTo()
	{
		return null;
	}

static final int[][] ykvMac7tbl =  // [14][4]
{
    { 0x0030,0x002E,     0,     0 }, // 0x8591  # digit zero full stop
    { 0x0058,0x0049,0x0049,0x0049 }, // 0x85AB  # roman numeral thirteen
    { 0x0058,0x0049,0x0056,     0 }, // 0x85AC  # roman numeral fourteen
    { 0x0058,0x0056,     0,     0 }, // 0x85AD  # roman numeral fifteen
    { 0x0078,0x0069,0x0069,0x0069 }, // 0x85BF  # small roman numeral thirteen
    { 0x0078,0x0069,0x0076,     0 }, // 0x85C0  # small roman numeral fourteen
    { 0x0078,0x0076,     0,     0 }, // 0x85C1  # small roman numeral fifteen
    { 0x0054,0x0042,     0,     0 }, // 0x865D  # square TB
    { 0x2193,0x2191,     0,     0 }, // 0x86CE  # down arrow left up arrow
    { 0x0028,0x5927,0x0029,     0 }, // 0x8791  # ideograph big + CIRCLE
    { 0x0028,0x5C0F,0x0029,     0 }, // 0x8792  # ideograph small + CIRCLE
    { 0x0028,0x63A7,0x0029,     0 }, // 0x879D  # ideograph memo + CIRCLE
    { 0x6709,0x9650,0x4F1A,0x793E }, // 0x87FB  # square "[yuugen gaisha]"
    { 0x8CA1,0x56E3,0x6CD5,0x4EBA }  // 0x87FC  # square "[zaidan houjin]"
};

}

//
// end of program
//
